define("de_epages/spreedly/ui/checkoutconfirmation", [
    "jquery",
    "de_epages/spreedly/ui/checkoutlifecycle",
    "spreedlyexpress2",
    "ep/ajax",
    "ep/fn/busy",
    "$ready!"
], function ($, checkoutlifecycle) {

    return function(
    SpreedlyEnvironmentKey,
    BasketTotalAmount,
    ShopName,
    BasketId,
    IsMobile
    ){
        var SpeedlyExpressWidgetDisplayOptions = {
            "amount": BasketTotalAmount,
            "company_name": ShopName
        };

        var BuyButton = IsMobile === "true" ?
                ep(".OrderProcessNavigation button") : ep("div.Confirm-Order button.SubmitButton");

        var BasketForm = IsMobile === "true" ?
                ep("form#BasketOfferForm") : ep("form#BasketForm");

        function validateBasketFormRequiredCheckBoxes() {
            if(IsMobile === "true") {
                var CheckBoxValidation = true;
                var TermsAndConditionsCheckBox = ep("form#BasketOfferForm input#AcceptTAC");
                if(TermsAndConditionsCheckBox.length > 0) {
                    if(TermsAndConditionsCheckBox.prop("checked") == false) {
                        TermsAndConditionsCheckBox.parent("div").addClass("DialogError");
                        CheckBoxValidation = false;
                    } else {
                        TermsAndConditionsCheckBox.parent("div").removeClass("DialogError");
                    }
                }

                var AcceptCustomerInformation = ep("form#BasketOfferForm input#AcceptCustomerInformation");
                if(AcceptCustomerInformation.length > 0) {
                    if(AcceptCustomerInformation.prop("checked") == false) {
                        AcceptCustomerInformation.parent("div").addClass("DialogError");
                        CheckBoxValidation = false;
                    } else {
                        AcceptCustomerInformation.parent("div").removeClass("DialogError");
                    }
                }
                return CheckBoxValidation;
            } else {
                BasketForm.trigger("validate");
                return (ep("form#BasketForm .ui-invalid").length === 0);
            }
        }

        function getParameterValue(parameterName) {
            var result = null;
            var tmp = [];
            var items = location.search.substr(1).split("&");
            for (var index = 0; index < items.length; index++) {
                tmp = items[index].split("=");
                if (tmp[0] === parameterName) result = decodeURIComponent(tmp[1]);
            }
            return result;
        }

        function recoverMarkedCheckboxes() {
            if ($('#AcceptTAC').length) {
                $('#AcceptTAC').prop("checked", true);
            }
            if ($('#AcceptCustomerInformation').length) {
                $('#AcceptCustomerInformation').prop("checked", true);
            }

            if (getParameterValue("RatingInvitationFlag") === "true") {
                $('#IsRatingInvitationAllowed').prop("checked", true);
            }
        }

        function handleSpreedlyCallback() {
            if (getParameterValue("transaction_token") && "ViewMultiCheckoutConfirmation" === getParameterValue("ViewAction")) {
                ep("body").busy("show");
                recoverMarkedCheckboxes();
                // use document element because at 3DS1 redirection, BasketForm selector does not work.
                document.forms["BasketForm"].submit();
            } else {
                onBuyButtonConfirmation();
            }
        }

        function submitBasketForm() {
            BasketForm.submit();
        }

        function appendErrorAndSubmitBasketForm(Error) {
            if(Error && Error.Name) {
                BasketForm.append("<input type='hidden' value='" + Error.Name + "' name='ErrorName'>");
            }
            if(Error && Error.Reason) {
                BasketForm.append("<input type='hidden' value='" + Error.Reason + "' name='ErrorReason'>");
            }
            submitBasketForm();
        }

        function onSuccessfulPurchaseResponse(Transaction) {
            if (Transaction.state === "succeeded") {
                return submitBasketForm();
            } else if (Transaction.state === "error") {
                appendErrorAndSubmitBasketForm({
                    "Name": "Purchase",
                    "Reason": "CardDeclined",
                });
            } else if (Transaction.state === "pending") {
                if (Transaction.required_action === "redirect") {
                    on3DS1Fallback(Transaction);
                } else {
                    checkoutlifecycle(
                        Transaction.state,
                        Transaction.token,
                        Transaction.required_action,
                        Transaction.device_fingerprint_form,
                        Transaction.challenge_url,
                        Transaction.challenge_form,
                        Transaction.checkout_form,
                        Transaction.checkout_url,
                        BasketId,
                        submitBasketForm,
                        appendErrorAndSubmitBasketForm,
                        SpreedlyEnvironmentKey
                    );
                }
            } else {
                appendErrorAndSubmitBasketForm({
                    "Name": "Purchase",
                    "Reason": "UnknownError",
                });
            }
        }

        function on3DS1Fallback(Transaction) {
            ep("body").busy("show");
            if (Transaction.checkout_form) {
                $(Transaction.checkout_form).appendTo("body").submit();
            } else if (Transaction.checkout_url) {
                window.location.replace(Transaction.checkout_url);
            } else {
                appendErrorAndSubmitBasketForm({
                    "Name": "Purchase",
                    "Reason": "UnknownError",
                });
            }
        }

        function getPurchaseRequestData() {
            var SpreedlyWidgetFullscreenSize = '05';
            var requestData = {
                "ChangeAction": "MakeSpreedlyPurchase",
                "ObjectID": BasketId,
                "SpreedlyBrowserInfo": Spreedly.ThreeDS.serialize(SpreedlyWidgetFullscreenSize)
            };
            if ($('#IsRatingInvitationAllowed').prop("checked")) {
                requestData["IsRatingInvitationAllowed"] = 1;
            }
            if ($('#GatewayFieldBirthday').length) {
                requestData["GatewayFieldBirthday"] = $('#GatewayFieldBirthday').val();
            }
            return requestData;
        }

        function onBuyButtonClick() {
            if (!validateBasketFormRequiredCheckBoxes()) {
                ep("body").busy("hide");
                return false;
            }
            var purchaseRequestData = getPurchaseRequestData();

            ep.ajax({
                "data" : purchaseRequestData,
                "type" : "post",
                "dataType" : "json"
            }).done(function(response) {
                if(!response.Error && response.Transaction){
                    onSuccessfulPurchaseResponse(response.Transaction);
                } else {
                    appendErrorAndSubmitBasketForm(response.Error);
                }
            }).fail(function( response ){
                appendErrorAndSubmitBasketForm({
                    "Name": "Purchase",
                    "Reason": "ConnectionError",
                });
            });
        }

        function onBuyButtonConfirmation() {
            BuyButton.on('click.submitSpreedlyPurchase', function(event) {
                event.stopPropagation();
                event.preventDefault();
                ep("body").busy("show");
                onBuyButtonClick();
                return false;
            });
            SpreedlyExpress.init(SpreedlyEnvironmentKey, SpeedlyExpressWidgetDisplayOptions);
            SpreedlyExpress.onInit(function(){
                BuyButton.prop("disabled", false);
            });
        }


        BuyButton.prop("disabled", true);

        $(document).ready(function() {
            handleSpreedlyCallback();
        });
    };
});
